# coding: latin1
"""
  Modulo per creare file Excel formato Foglio XML (Excel 2003)
  Pu essere usato per creare un file completamente nuovo o per aggiungere WorkSheet
  o Stili ad un file Excel precedentemente salvato nello stesso formato (ottenendo un nuovo file)

  La logica d'uso  la seguente:

  Istanziare se necessario una o pi strutture di tipo
    xlsSheetStruct            per definire uno o pi fogli con caratteristiche diverse dal default
                              (qui vi sono i margini, l'intestazione, il pi di pagina, ecc.)
    xlsStyleStruct            per definire uno o pi stili di cella diversi dal default
                              (qui vi sono colori, bordatura, font, ecc. della cella con questo stile)
  e variarne le propiet, da usare successivamente

  Istanziare
  - xlsExcel([NomeFile], [Autore], [SheetStruct])
    per inizializzare il file e preparare i tre stili di default (txt, num e date)
    Se il nome file manca, il file xml viene creato in memoria e (dopo xlsClose) pu
    essere recuperato con la propiet xlsOut
    SheetStruct  una variabile precedentemente istanziata di tipo xlsSheetStruct che rappresenta
    il foglio di default

  Richiamare i metodi (in quest'ordine):
  - xlsAddStyle(StyleStruct, [IDstile])
    lanciata opzionalmente (anche pi volte) per aggiungere gli stili aggiuntivi voluti delle celle
    StyleStruct  una variabile precedentemente istanziata di tipo xlsStyleStruct
    IDstile  l'ID della struttura, che viene richiamato alla scrittura dei dati; se non  presente
    deve essere assegnato all'interno della struttura stessa

  - xlsAddSheet([nomeFoglio], [SheetStruct])
    inizializza la scrittura di un nuovo foglio
    SheetStruct  una variabile precedentemente istanziata di tipo xlsSheetStruct
    Se nomeFoglio non  fornito, deve essere presente all'interno della struttura
    (Pi chiamate permetteranno di scrivere pi fogli Excel nello stesso file)

    A questa funzione devono seguire una o pi chiamate alla funzione xlsColumn() (se si vogliono
    differenziare le colonne tra loro) e poi
       xlsWriteCell() che scrive materialmente i dati del foglio in esame, in sequenza da sinistra
    a destra e dall'alto al basso, finch
     - o si richiama nuovamente xlsAddSheet per chiudere il precedente e aprire un nuovo foglio
     - o si chiude tutto con xlsClose()

  - xlsColumn(numColonna, Larghezza, [IDstile])
    imposta la Larghezza in visualizzazione di ciascuna colonna ed eventualmente lo stile;
    deve seguire immediatamente la funzione xlsAddSheet()
    le colonne specificate devono essere in ordine crescente a partire da 1, eventualmente
    saltando quelle in cui si vuole lasciare i default
    La Larghezza da impostare  circa uguale al numero di caratteri * 4 (con i caratteri di default)

  - xlsWriteCell(Valore, [Tipo, IDstile, formula, skipCol, MergeAcross, MergeDown])
    scrittura del dato nella cella; le celle devono essere in ordine crescente dentro la stessa riga
    Tipo pu essere "S" (Stringa, di default), "N" (Numero) e "D" (Data, non ancora gestito);
    Valore in input pu essere un unico valore o una lista di valori, nel qual caso ciascuno
    andr in una cella distinta della stessa riga
    IDstile deve essere tra quelli definiti, altrimenti avr uno stile di default; se  = None non gli
    verr assegnato alcun stile e quindi prender lo stile della riga, della colonna o del tipo di dato.
    Se MergeAcross > 0, indica che la cella si estende anche ad una o pi celle successive a destra
    Se MergeDown > 0, indica che la cella si estende anche ad una o pi celle successive in basso

  - xlsNewLine([skip], [RowWidth], [IDStile])
    passa alla riga successiva (o alla successiva + skip); la nuova riga pu avere la
    larghezza impostata a RowWidth e lo stile = IDStile

  - PosizioneCella = xlsGetPosition()
    restituisce le coordinate assolute della cella relativa attuale come stringa (per l'uso nelle formule):
    PosizioneCella  la posizione nella forma Excel "Lettera" + "numRiga" (ossia colonna + riga)
    Riga, Colonna attuali (in formato numerico) si possono ottenere dalle variabili di istanza:
       ActualRow, ActualCol

  - Rettangolo = xlsGetRectangle(R1, C1, R2, C2)
    date in input le posizioni assolute Riga1+Colonna1, Riga2+Colonna2, che definiscono un rettangolo di celle,
    viene restituita la stringa che definisce la posizione relativa del rettangolo rispetto la cella attuale
    Serve come aiuto per definire le formule

  - xlsClose()
    deve essere l'ultima istruzione che chiude il file


  Se si vuole fare il Merge con un altro file Excel esistente in formato Foglio XML:

  Istanziare
  xlsExcel([NomeFile], OldExcelFile="NomeFileOld")
    Viene generato un file che contiene i fogli vecchi presi dal file "NomeFileOld" pi quelli nuovi generati.
    A questa funzione seguono le altre nel solito modo, in cui gli stili che sono gi presenti con lo stesso ID
    non vengono aggiunti se scritti da programma e gli sheet generati vengono aggiunti in fondo a quelli gi presenti

  Richiamare xlsSheetDelete(nomeFoglio)
    nel caso di Merge, nel file generato cancella il foglio indicato; deve essere usata (anche pi volte)
    prima di generare il primo sheet
    Deve essere richiamata subito dopo xlsExcel e prima di xlsAddSheet

"""
from string import Template
from datetime import datetime

class xlsSheetStruct:
   """ Struttura di definizione di un foglio """
   def __init__(self, NomeSheet="NoName"):
      self.SheetName    = NomeSheet     # nome breve senza spazi
      self.Description  = ""
      self.FootPage     = ""
      self.HeaderMargin = 0.5           # in Inch, default = "0.5"
      self.FooterMargin = 0.5           # in Inch, default = "0.5"
      self.MarginTop    = 1             # in Inch, default = "1", anche numero decimale
      self.MarginBottom = 1             # in Inch
      self.MarginLeft   = 1             # in Inch
      self.MarginRight  = 1             # in Inch
      self.PrintGrid    = True          # se = "1", in stampa visualizza la griglia (default = "1")
      self.Protected    = False         # se = "1" protegge il Foglio, altrimenti = "0" (default)
      self.RowHeight    = 0             # altezza di default delle righe (0 = non gestita, 16 = normale)
      self.Layout       = "L"           # pu essere 'P' (portrait) o 'L' (landscape)

class xlsStyleStruct:
   """ Struttura di definizione di uno stile di cella """
   def __init__(self, IDstile=""):
      self.ID           = IDstile         # ID senza spazi che identifica lo Style
      # Allineamento del testo contenuto nella cella
      self.Align        = "Left"          # "Left" (default), "Right", "Center"
      self.VAlign       = "Bottom"        # "Top", "Bottom" (default), "Center"
      self.Indent       = 0               # numero di caratteri di rientro del testo
      # Bordo della cella
      self.BorderType   = "Continuous"    # (=LineStyle) di default  "Continuous", pu essere anche:
                                          #  "Double", "Dot", "Dash", "DashDot"
      self.BorderColor  = ""              # di default  "#000000" = nero
      self.BorderTop    = 0               # 0 = nessun bordo in alto, altrimenti valore da 1 a 3 (= Weight)
      self.BorderBottom = 0               # 0 = nessun bordo in basso, altrimenti da 1 a 3
      self.BorderLeft   = 0               # 0 = nessun bordo a sinistra, altrimenti da 1 a 3
      self.BorderRight  = 0               # 0 = nessun bordo a destra, altrimenti da 1 a 3
      # Caratteristiche del Font usato nella cella
      self.FontName     = ""              # nome del font, se nessuno,  il font di default
      self.FontBold     = False
      self.FontItalic   = False
      self.FontSize     = 0
      self.FontColor    = ""              # di default  "#000000" = nero
      # Caratteristiche dello sfondo della cella (Interior)
      self.BGcolor      = ""              #
      self.BGpattern    = "Solid"         # nome del pattern di colorazione, di default "Solid", pu essere anche, tra gli altri:
                                          # "Gray50", "Gray25", "Gray125", "Gray0625", "ThinDiagStripe", "ThinReverseDiagStripe",
                                          # "ThinDiagCross", "ThinHorzCross", "ThinHorzStripe", "ThinVertStripe",
      self.BGpatternColor  = ""           #
      self.NumberFormat = ""              # di default niente, pu essere "Fixed", "Scientific", "Short Date"
      self.WrapText     = False           # di default niente, pu essere "1" per andare a capo nella stessa cella
      self.VerticalText = False           # di default niente, pu essere "1" per testo scritto verticalmente
      self.Rotate       = 0               # di default nessuna rotazione, altrimenti  l'angolo in gradi, anche negativo
      self.Protected    = False           # se = "1" protegge la cella, altrimenti = "0",
      # i colori vanno usati come stringhe RGB esadecimali, del tipo "#rrggbb", oppure usando
      # il nome del colore inglese in minuscolo

class xlsExcel:
   """ Crea intestazione XML e prepara stili di default """
   Alfabeto = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"    # per la decodifica colonna-riga

   def __init__(self, FileName="", Author="ExcelExport", Sheet="", OldExcelFile=""):
      self.FileName = FileName.strip()
      self.Author   = Author
      # Istanzia la struttura Sheet di default
      if Sheet == "":
         self.__defaultSheet =  xlsSheetStruct()
      else:
         if type(Sheet).__name__ != 'instance': raise TypeError, "Non  una istanza di struttura di FoglioExcel"
         self.__defaultSheet    = Sheet

      # Elementi fissi, contenenti delle variabili da valorizzare
      self.xmlHead = """<?xml version="1.0" encoding="utf-8" standalone="yes" ?>
<?mso-application progid="Excel.Sheet"?>
<Workbook xmlns="urn:schemas-microsoft-com:office:spreadsheet" xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet" xmlns:html="http://www.w3.org/TR/REC-html40">
 <DocumentProperties xmlns="urn:schemas-microsoft-com:office:office">
  <LastAuthor>${AUTHOR}</LastAuthor>
  <Created>${DATEXML}</Created>
  <Version>11.0000</Version>
 </DocumentProperties>
 <ExcelWorkbook xmlns="urn:schemas-microsoft-com:office:excel">
  <ProtectStructure>False</ProtectStructure>
  <ProtectWindows>False</ProtectWindows>
 </ExcelWorkbook>
 <Styles>
  <Style ss:ID="Default" ss:Name="Normal">
   <Alignment ss:Vertical="Bottom"/>
   <Borders/>
   <Font/>
   <Interior/>
   <NumberFormat/>
   <Protection/>
  </Style>
  <Style ss:ID="txt">
   <Alignment ss:Horizontal="Left" ss:Vertical="Bottom"/>
   <NumberFormat/>
   <Protection ss:Protected="0"/>
  </Style>
  <Style ss:ID="num">
   <Alignment ss:Horizontal="Right" ss:Vertical="Bottom"/>
   <NumberFormat/>
   <Protection ss:Protected="0"/>
  </Style>
  <Style ss:ID="date">
   <Alignment ss:Horizontal="Center" ss:Vertical="Bottom"/>
   <NumberFormat ss:Format="Short Date"/>
   <Protection ss:Protected="0"/>
  </Style>"""
      self.xmlTail = "</Workbook>"
      dt = str(datetime.now())
      dt = dt.replace(' ', 'T')[:19] + 'Z'
      # Flag e Contatori vari:
      self.nSheets = 0
      self.listaSheet = []  # per controllo sui doppioni
      self.listaStili = []  # per controllo sui doppioni
      self.SheetsToDel =[]  # per eventuale elenco Sheet da cancellare quando si fa il Merge
      self.ActualRow = 1    # contatore righe
      self.ActualCol = 1    # contatore colone
      self.__Sequenza = 0   # 0 = inizio (si possono aggiungere stili), 1 =  iniziato uno sheet, 2 =  iniziata la scrittura celle
      self.__flagOpenRow = False
      self.__RowAttribute = ""

      # Se c' "OldExcelFile", bisogna fare il merge con un file Excel esistente
      if OldExcelFile != "" :
         self.flagMerge = True
         self.xmlOut = []
         self.xmlOutOld = []
         self.fObjOldExcel = open(OldExcelFile, "r")
         self.__InitMergeSheet(OldExcelFile)
      else:   # altrimenti inizia a scrivere il nuovo file
         self.flagMerge = False
         # L'output viene accumulato nell'array xmlOut in cui ogni elemento contiene una o pi righe
         self.xmlOut = [Template(self.xmlHead).safe_substitute(AUTHOR=Author,DATEXML=dt)]
         self.listaStili = ['Default', 'num', 'txt', 'date']  # per controllo sui doppioni


   def xlsAddStyle(self, Stile, IDstile=""):
      """ Aggiunge uno Style nella sezione <Styles> """
      # Controlli
      if type(Stile).__name__ != 'instance': raise TypeError, "Non c' in input una istanza di struttura di stile"
      if IDstile.strip() == "" and Stile.ID.strip() == "": raise Exception, "Manca l'ID dello stile"
      if IDstile.strip() != "": Stile.ID = IDstile
      if self.__Sequenza != 0: raise Exception, "non si pu aggiungere uno stile a questo punto"
      for e in self.listaStili:
          if e == Stile.ID: raise Exception, "ID stile '" + Stile.ID + "' duplicato"
      self.listaStili.append(Stile.ID)
      # Inizia ad aggiungere lo stile
      ActualStyle = "  <Style ss:ID=\"" + Stile.ID + "\">\n"
      # Allineamento
      if Stile.Align == "": Stile.Align = "Left"            # "Left" (default), "Right", "Center"
      if Stile.VAlign == "": Stile.VAlign = "Bottom"        # "Top", "Bottom" (default), "Center"
      if Stile.Indent > 0:
          s += " ss:Indent=\"" + str(Stile.Indent) + "\""
      if Stile.WrapText:
          s = " ss:WrapText=\"1\""
      else:
          s = ""
      if Stile.VerticalText :
          s += " ss:VerticalText=\"1\""
      if Stile.Rotate != 0 :
          s += " ss:Rotate=\"" + str(Stile.Rotate) + "\""
      ActualStyle += "   <Alignment ss:Horizontal=\"" + Stile.Align + "\" ss:Vertical=\"" + Stile.VAlign + "\"" + s +"/>\n"
      # bordo della cella
      if Stile.BorderTop != 0 or Stile.BorderBottom != 0 or Stile.BorderLeft != 0 or Stile.BorderRight != 0:
         s = "   <Borders>\n"
         if Stile.BorderType == "" : Stile.BorderType = "Continuous"
         if Stile.BorderTop != 0 :
            s = s + "    <Border ss:Position=\"Top\" ss:LineStyle=\"" + Stile.BorderType + "\" ss:Weight=\"" + str(Stile.BorderTop) + "\"/>"
         if Stile.BorderBottom != 0 :
            s = s + "    <Border ss:Position=\"Bottom\" ss:LineStyle=\"" + Stile.BorderType + "\" ss:Weight=\"" + str(Stile.BorderBottom) + "\"/>"
         if Stile.BorderLeft != 0 :
            s = s + "    <Border ss:Position=\"Left\" ss:LineStyle=\"" + Stile.BorderType + "\" ss:Weight=\"" + str(Stile.BorderLeft) + "\"/>"
         if Stile.BorderRight != 0 :
            s = s + "    <Border ss:Position=\"Right\" ss:LineStyle=\"" + Stile.BorderType + "\" ss:Weight=\"" + str(Stile.BorderRight) + "\"/>"
         # Stile.BorderColor        # di default  "#000000" = nero
         ActualStyle +=  s + "   </Borders>\n"

      # Caratteristiche del Font usato nella cella
      if Stile.FontName != "" or Stile.FontBold or Stile.FontItalic or Stile.FontSize != 0 or Stile.FontColor != "":
         s = "   <Font "
         if Stile.FontName != "":
            s = s + "ss:FontName=\"" + Stile.FontName + "\" "
         if Stile.FontBold:
            s = s + "ss:Bold=\"1\" "
         if Stile.FontItalic:
            s = s + "ss:Italic=\"1\" "
         if Stile.FontSize != 0:
            s = s + "ss:Size=\"" + str(Stile.FontSize) + "\" "
         if Stile.FontColor != "":
            s = s + "ss:Color=\"" + Stile.FontColor + "\" "
         ActualStyle +=  s + "/>\n"

      # Caratteristiche dello sfondo della cella (Interior)
      if Stile.BGcolor != "" or Stile.BGpattern != "":
         if Stile.BGpattern == "": Stile.BGpattern = "Solid"
         s = "   <Interior"
         if Stile.BGcolor != "": s = s + " ss:Color=\"" + Stile.BGcolor + "\""
         if Stile.BGpattern != "": s = s + " ss:Pattern=\"" + Stile.BGpattern + "\""
         if Stile.BGpatternColor != "": s = s + " ss:PatternColor=\"" + Stile.BGpatternColor + "\""
         ActualStyle +=  s + "/>\n"
      if Stile.NumberFormat != "":
         ActualStyle += "   <NumberFormat ss:Format=\"" + Stile.NumberFormat + "\"/>\n"
      else:
         ActualStyle += "   <NumberFormat/>\n"
      if Stile.Protected:
         ActualStyle += "   <Protection ss:Protected=\"1\"/>\n"
      else:
         ActualStyle += "   <Protection ss:Protected=\"0\"/>\n"

      ActualStyle += "  </Style>"

      self.xmlOut.append(ActualStyle)

   def xlsAddSheet(self, nomeSheet="", Sheet=""):
      """ Aggiunge un foglio Excel, chiudendo eventualmente il precedente e aprendo il nuovo """
      # Controlli
      if Sheet == "": Sheet = self.__defaultSheet
      if type(Sheet).__name__ != 'instance': raise TypeError, "Non c' in input una istanza di struttura di sheet"
      if Sheet.SheetName.strip() == "" and nomeSheet.strip() == "": raise Exception, "Manca il nome dello sheet"
      if nomeSheet.strip() != "": Sheet.SheetName = nomeSheet
      for e in self.listaSheet:
         if e == Sheet.SheetName: raise Exception, "Nome Sheet '" + Sheet.SheetName + "' duplicato"

      self.listaSheet.append(Sheet.SheetName)
      if self.__Sequenza > 0:     # chiude lo sheet precedente
         self.__ChiudiSheet()
      else:                       # chiude gli Styles, perch questo  il primo sheet
         self.xmlOut.append(" </Styles>")
         # Se c' "OldExcelFile", bisogna ricopiare i fogli esistenti nel vecchio file
         if self.flagMerge :  self.__CopiaSheet()

      if Sheet.SheetName == "": Sheet.SheetName = "Sheet" + self.nSheets
      if Sheet.Description == "":
          Sheet.Description = "Foglio Excel '" + Sheet.SheetName + "' generato automaticamente"
      else:
          Sheet.Description = Sheet.Description.decode('latin1').encode('utf8')
      if Sheet.FootPage == "":
          Sheet.FootPage = "Pagina &amp;P"
      else:
          Sheet.FootPage = Sheet.FootPage.decode('latin1').encode('utf8')
      # Imposta i default
      if Sheet.MarginTop == 0:
         Sheet.MarginTop = 0.1    # stringa numerica in Inch, default = "0.1"
      self.__Sheet = Sheet        # Salva lo sheet per la chiusura
      self.nSheets += 1

      # Apre il nuovo Sheet
      ActualSheet = " <Worksheet ss:Name=\"" + Sheet.SheetName + "\">\n" + \
        "  <Table ss:ExpandedColumnCount=\"256\" x:FullColumns=\"1\" x:FullRows=\"1\">"
      self.xmlOut.append(ActualSheet)
      # Inizialzza variabili di appoggio
      self.ActualRow = 1
      self.ActualCol = 1
      self.__flagOpenRow = False
      self.__RowAttribute = ""
      self.__Sequenza = 1

   def __ChiudiSheet(self):
      " Genera l'XML di chiusura Sheet "

      s1 = ""
      if self.__Sheet.HeaderMargin != 0:
          s1 = "x:Margin=\"" + str(self.__Sheet.HeaderMargin) + "\" "
      s2 = ""
      if self.__Sheet.FooterMargin != 0:
          s2 = "    <Footer x:Margin=\"" + str(self.__Sheet.FooterMargin) + "\" x:Data=\"" + self.__Sheet.FootPage + "\"/>\n"
      s3 = ""
      if self.__Sheet.Layout == "L":
          s3 = "    <Layout x:Orientation=\"Landscape\"/>\n"
      if self.__Sheet.Layout == "P":
          s3 = "    <Layout x:Orientation=\"Portrait\"/>\n"
      s4 = ""
      if self.__Sheet.PrintGrid:
          s4 = "    <Gridlines/>\n"
      if self.__flagOpenRow:   # se c' una riga aperta, la chiude
          self.xmlOut.append("   </Row>")
          self.__flagOpenRow = False

      # Genera l'XML di chiusura Sheet
      ActualSheet = \
        "  </Table>\n"  + \
        "  <WorksheetOptions xmlns=\"urn:schemas-microsoft-com:office:excel\">\n" + \
        "   <PageSetup>\n" + s3 + \
        "    <Header " + s1 + "x:Data=\"" + self.__Sheet.Description + "\"/>\n" + s2 + \
        "    <PageMargins x:Bottom=\"" + str(self.__Sheet.MarginBottom) + "\" x:Left=\"" + str(self.__Sheet.MarginLeft) + \
        "\" x:Right=\"" + str(self.__Sheet.MarginRight) + "\" x:Top=\"" + str(self.__Sheet.MarginTop) + "\"/>\n" + \
        "   </PageSetup>\n" + \
        "   <Print>\n" + s4 + \
        "   </Print>\n" + \
        "   <Selected/>\n" + \
        "   <ProtectObjects>False</ProtectObjects>\n" + \
        "   <ProtectScenarios>False</ProtectScenarios>\n" + \
        "  </WorksheetOptions>\n" + \
        " </Worksheet>"
      self.xmlOut.append(ActualSheet)

   def xlsColumn(self, numColonna, Larghezza, IDstile=""):
      """ Cambia le caratteristiche di default di una colonna, come larghezza e stile
          La Larghezza da impostare  circa uguale al numero di caratteri * 4 (con i caratteri di default)"""
      # Controlli
      if numColonna < 1 or numColonna > 256: raise Exception, "Numero di colonna errato"
      if self.__Sequenza != 1: raise Exception, "Non si pu aggiungere una definizione Colonna a questo punto"
      if IDstile != "":   # controlla che lo stile sia definito
         for e in self.listaStili:
            if e == IDstile:
               ok = True
               break
         if not ok:  raise Exception, "ID stile '" + IDstile + "' errato o mancante."
      s = ""
      if IDstile.strip() != "": s = "ss:StyleID=\"" + IDstile.strip() + "\" "

      ActualColumn = "   <Column ss:Index=\"" + str(numColonna) + "\" " + s + \
         "ss:AutoFitWidth=\"0\" ss:Width=\"" + str(Larghezza) + "\"/>"
      self.xmlOut.append(ActualColumn)

   def xlsWriteCell(self, Valore, Tipo="S", IDstile="", Formula="", nColSkip=0, MergeAcross=0, MergeDown=0):
      """ Scrive nella cella attuale un dato di tipo Stringa, Numero o Data """
      # Controlli: Tipo pu essere: "N", "D" o "S"
      if self.__Sequenza < 1: raise Exception, "Non si pu scrivere una cella a questo punto"
      if IDstile is not None and IDstile != "":   # controlla che lo stile sia definito
         for e in self.listaStili:
            if e == IDstile:
               ok = True
               break
         if not ok:  raise Exception, "ID stile '" + IDstile + "' errato o mancante."
      if nColSkip < 0 or nColSkip > 200: nColSkip = 0

      if Tipo.strip() == "N" :
         s1 = "Number"
         s4 = "ss:StyleID=\"num\""
      elif Tipo.strip() == "D":
         s1 = "DateTime"
         s4 = "ss:StyleID=\"date\""
      else:
         Tipo = "S"
         s1 = "String"
         s4 = "ss:StyleID=\"txt\""

      sm = ""
      if  MergeAcross > 0: sm = "ss:MergeAcross=\"" + str(MergeAcross) + "\" "
      if  MergeDown > 0: sm = "ss:MergeDown=\"" + str(MergeDown) + "\" "

      # Apre la eventuale nuova riga
      if not self.__flagOpenRow:
         self.xmlOut.append("   <Row" + self.__RowAttribute + ">")
         self.__flagOpenRow = True
      # si posiziona nella colonna giusta
      self.ActualCol += nColSkip
      s2 = ""
      if nColSkip > 0:
         s2 = "ss:Index=\"" + str(self.ActualCol) + "\" "

      s3 = ""
      if Formula != "": s3 = "ss:Formula=\"" + Formula + "\" "
      if IDstile is None:  # toglie lo stile
         s4 = ""
      elif IDstile != "":
         s4 = "ss:StyleID=\"" + IDstile + "\" "

      t = type(Valore).__name__
      # Se "Valore"  una lista, mette in output tutti i suoi componenti
      if t == 'list':
         ActualSheet = ""
         for e in Valore:
            if type(e).__name__ != 'str':   # se i componenti non sono stringhe, li converte
               v = str(e)
            else:
               v = e
               v = v.decode('latin1').encode('utf8')
               v = v.replace("&", "&amp;")
               v = v.replace("<", "&lt;")
               v = v.replace(">", "&gt;")
            if ActualSheet != "" :   # aggiunge a capo tranne l'ultimo campo
               ActualSheet += "\n"
               self.ActualCol += 1
            ActualSheet += "    <Cell " + sm + s2 + s3 + s4 + "><Data ss:Type=\"" + s1 + "\">" + \
               v + "</Data></Cell>"
      else:
         if type(Valore).__name__ != 'str': # se il Valore non  una stringa, la converte
            v = str(Valore)
         else:
            v = Valore
            v = v.decode('latin1').encode('utf8')
            v = v.replace("&", "&amp;")
            v = v.replace("<", "&lt;")
            v = v.replace(">", "&gt;")
         ActualSheet = "    <Cell " + sm + s2 + s3 + s4 + "><Data ss:Type=\"" + s1 + "\">" + \
               v + "</Data></Cell>"
      self.xmlOut.append(ActualSheet)
      self.ActualCol += 1
      self.__RowAttribute = ""
      self.__Sequenza = 2


   def xlsNewLine(self, skip=0, RowWidth=0, IDstile=""):
      """ Passa alla riga successiva del foglio Excel """
      # Controlli
      if self.__Sequenza < 1: raise Exception, "Non si pu saltare una linea a questo punto"

      # aggiorna i contatori riga-colonna
      self.ActualRow += skip + 1
      self.ActualCol = 1

      s1 = ""
      if skip > 0 and skip <= 200: s1 = " ss:Index=\"" + str(self.ActualRow) + "\""
      s2 = ""
      if RowWidth > 0 and RowWidth < 200: s2 = " ss:AutoFitHeight=\"0\" ss:Height=\"" + str(RowWidth) + "\""
      s3 = ""
      ok = False
      if IDstile != "":   # controlla che lo stile sia definito
         for e in self.listaStili:
            if e == IDstile:
               ok = True
               break
         if not ok:  raise Exception, "ID stile '" + IDstile + "' errato o mancante."
         s3 = " ss:StyleID=\"" + IDstile + "\""

      if self.__flagOpenRow:    # se la riga precedente  ancora aperta, la chiude
         self.xmlOut.append("   </Row>")
      self.__flagOpenRow = False
      self.__RowAttribute = s1 + s2 + s3

   def xlsClose(self):
      """ Chiusura XML ed eventuale output del file """
      if self.__Sequenza < 2: raise Exception, "Non si pu chiudere a questo punto"

      self.__ChiudiSheet()
      self.xmlOut.append(self.xmlTail)
      # Se c' nome file, vi scrive l'XML
      if self.FileName != "" :
         if self.FileName.find(".") <= 0 : self.FileName += ".xls"
         fObj = open(self.FileName, "w")
         fObj.write("\n".join(self.xmlOut))
         fObj.close()

   def xlsGetPosition(self):
      """ Ritorna la posizione attuale in forma di sigla Lettera+Numero """
      if self.ActualCol > 26:       # 26  il numero di lettere dell'alfabeto = Len(__Alfabeto)
         n = self.ActualCol // 26   # divisione intera
         r = self.ActualCol % 26    # resto
         if n > 26: return(None)
         s = self.Alfabeto[n] + self.Alfabeto[r]
      else:
         s = self.Alfabeto[self.ActualCol-1]
      return(s + str(self.ActualRow))

   def xlsGetRectangle(self, Riga1, Col1, Riga2, Col2):
      """ Ritorna la stringa che rappresenta il rettangolo di celle voluto, per usarlo nelle formule """
      s = "R"
      if self.ActualRow != Riga1:
         s = "R[" + str(Riga1-self.ActualRow) + "]"
      s = s + "C"
      if self.ActualCol != Col1:
         s = s + "[" + str(Col1-self.ActualCol) + "]"
      s = s + ":R"
      if self.ActualRow != Riga2:
         s = s + "[" + str(Riga2-self.ActualRow) + "]"
      s = s + "C"
      if self.ActualCol != Col2:
         s = s + "[" + str(Col2-self.ActualCol) + "]"
      return(s)

   def xlsSheetDelete(self, NomeSheet):
      """ Esclude uno Sheet tra quelli da copiare facendo il Merge con un Excel vecchio """
      if NomeSheet.strip() != "": self.SheetsToDel.append(NomeSheet)

   def __InitMergeSheet(self, OldFile):
      " Inizia la copia da un file Excel all'altro controllando quali stili sono presenti "
      linea = self.fObjOldExcel.readline()
      if linea[0:6] != "<?xml ": raise Exception, "Il file '" + OldFile + "' non sembra un file Excel XML valido"
      nStyles = 0
      ErrLog = ""
      n = 1        # conta le linee
      while linea != "":
         self.xmlOutOld.append(linea)   # memorizza la linea precedente
         linea = self.fObjOldExcel.readline()
         n += 1
         # controlla l'XML della seconda linea
         if n == 2 and linea[0:18] != "<?mso-application ":
            ErrLog = "Il file '" + OldFile + "' non sembra un file Excel valido"
            break
         # raccoglie gli stili presenti
         if linea.find("<Style ") >= 0:
            Posiz1 = linea.find(":ID=\"")
            if Posiz1 < 0:
               ErrLog = "File in input con tag <Style> con ID mancante."
               break
            Posiz1 += 5
            Posiz2 = linea.find("\"", Posiz1)
            if Posiz2 < 0 :
               ErrLog = "File in input con tag <Style> con ID malformato."
               break
            # Estrae l'ID dello Style e lo aggiunge ai precedenti
            self.listaStili.append(linea[Posiz1:Posiz2])
            nStyles += 1
         # Esce a fine </Styles> senza scrivere questa linea su self.xmlOut
         if linea.find("</Styles>") > 0 : break

      if ErrLog.strip() != "": raise Exception, ErrLog
      self.xmlOut.append("".join(self.xmlOutOld))

      # Scrive gli Style di default se non sono gi stati scritti
      Trovato = False
      for e in self.listaStili:
         if e == "txt" :
             Trovato = True
             break
      if not Trovato:
         self.xmlOut.append( \
         "  <Style ss:ID=\"txt\">\n" + \
         "   <Alignment ss:Horizontal=\"Left\" ss:Vertical=\"Bottom\"/>\n" + \
         "   <NumberFormat/>\n   <Protection ss:Protected=\"0\"/>\n" + \
         "  </Style>")
         self.listaStili.append("txt")
         nStyles += 1

      Trovato = False
      for e in self.listaStili:
         if e == "num" :
             Trovato = True
             break
      if not Trovato:
         self.xmlOut.append( \
         "  <Style ss:ID=\"num\">\n" + \
         "   <Alignment ss:Horizontal=\"Right\" ss:Vertical=\"Bottom\"/>\n" + \
         "   <NumberFormat/>\n   <Protection ss:Protected=\"0\"/>\n" + \
         "  </Style>")
         self.listaStili.append("num")
         nStyles += 1

      Trovato = False
      for e in self.listaStili:
         if e == "date" :
             Trovato = True
             break
      if not Trovato:
         self.xmlOut.append( \
         "  <Style ss:ID=\"date\">\n" + \
         "   <Alignment ss:Horizontal=\"Center\" ss:Vertical=\"Bottom\"/>\n" + \
         "   <NumberFormat ss:Format=\"Short Date\"/>\n   <Protection ss:Protected=\"0\"/>\n" + \
         "  </Style>")
         self.listaStili.append("date")
         nStyles += 1


   def __CopiaSheet(self):
      " Copia da un file Excel all'altro eliminando eventualmente degli sheet "
      self.xmlOutOld = []
      flagSkip = False
      ErrLog = ""
      linea = self.fObjOldExcel.readline()
      n = 1
      while linea != "":
         # Esce a fine </Workbook> senza scrivere questa linea di chiusura
         if linea.find("</Workbook>") >= 0: break
         # raccoglie il nome degli sheet presenti, se  un inizio Sheet
         if linea.find("<Worksheet ") >= 0:
            Posiz1 = linea.find(":Name=\"")
            if Posiz1 < 0:
               ErrLog = "File in input con tag <Worksheet> con Name mancante."
               break
            Posiz1 += 7
            Posiz2 = linea.find("\"", Posiz1)
            if Posiz2 == 0:
               ErrLog = "File in input con tag <Worksheet> con Name malformato."
               break
            # Estrae il nome dello Sheet e lo aggiunge ai precedenti, se non  da saltare
            SheetsName = linea[Posiz1:Posiz2]
            flagSkip = False
            for e in self.SheetsToDel:
               if e == SheetsName :
                  flagSkip = True   # lo Sheet  da saltare
                  break

         if not flagSkip:
            self.xmlOutOld.append(linea)   # memorizza la linea appena letta

         linea = self.fObjOldExcel.readline()
         n += 1

      if ErrLog.strip() != "": raise Exception, ErrLog
      self.xmlOut.append("".join(self.xmlOutOld))

